${eCAP_AV_SDK_BOILERPLATE}

#ifndef ${eCAP_AV_SDK_GUARD}_ANTIVIRUS_H
#define ${eCAP_AV_SDK_GUARD}_ANTIVIRUS_H

#include <libecap/common/forward.h>
#include <string>

namespace Adapter {

class Answer;

// generic virus scanner with support for blocking and asynchronous scans
// kids implement scanning-specific methods
class Antivirus {
	public:
		class User {
			public:
				virtual ~User() {}

				// handles the scanning result
				virtual void onAnswer(const Answer &answer) = 0;
		};

		typedef libecap::Options Options;

	public:
		virtual ~Antivirus() {}

		virtual void configure(const Options &cfg) = 0;
		virtual void reconfigure(const Options &cfg) = 0;

		// refresh virus database, for example; does not change configuration
		virtual void update() = 0;

		// scan in the caller thread and call answer.deliver()
		void blockingScan(Answer &answer);

		// start scanning without blocking the caller
		// eventually "delivers" the answer via answer.deliver()
		// the answer object must be treated as invalid after this call
		void asyncScan(Answer *answer);

	protected:
		// scan answer.filename and update the answer
		// called from either a scanner or a host thread
		virtual void scan(Answer &answer) = 0;

		static void *AsyncScan(void *context);
};


// the result of scanning for a virus
// For now, the API is specific to scanners looking at files
class Answer
{
public:
	// summary of the scan status
	typedef enum {
		scUnknown, // scan has not started or has not finished
		scClean, // scan found nothing
		scVirus, // scan found a virus; see virusName
		scError // scan failed; see errorMsg
	} StatusCode;

	explicit Answer(const std::string &aFileName):
		fileName(aFileName),
		statusCode(scUnknown)
	{}

	virtual ~Answer() {}

	// send this answer, possibly asynchronously, to the requester
	virtual void deliver() = 0;

	const std::string fileName; // file to scan

	/* results of the scan */
	std::string virusName; // for scVirus
	std::string errorMsg; // for scError
	StatusCode statusCode; // overall outcome
};

} // namaspace Adapter

#endif
